<?php
// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}
/**
 * Class Strong_Testimonials_Pro_Addons
 *
 * @since 2.38
 */
class Strong_Testimonials_Pro_Addons {

	private $addons = array();
	private $site_host;
	private $license_key;
	public $plugins = array();

	/**
	 * The single instance of the class.
	 *
	 * @var Strong_Testimonials_Pro_Addons
	 */
	public static $instance = null;

	/**
	 * Strong_Testimonials_Pro_Addons constructor.
	 */
	private function __construct() {
		add_filter( 'wpmtst_addon_button_action', array( $this, 'output_download_link' ), 10, 2 );

		// Add script for installing addons
		add_action( 'admin_enqueue_scripts', array( $this, 'addons_scripts' ) );
		// Add ajax action in order to install our addons
		add_action( 'wp_ajax_wpmtst-install-addons', array( $this, 'install_addons' ), 20 );

		add_filter( 'st_plugins_upgrade_pro', array( $this, 'remove_plugins_upgrade_to_pro' ), 15, 1 );

		add_action( 'wpmtst_reload_extensions', array( $this, 'reload_extensions' ) );

		// Get website domain
		if ( function_exists( 'domain_mapping_siteurl' ) ) {
			$this->site_host = domain_mapping_siteurl( get_current_blog_id() );
		} else {
			$this->site_host = site_url();
		}

		// Get License key
		$this->license_key = trim( get_option( 'strong_testimonials_license_key' ) );
	}

	/**
	 * Get the instance of the class.
	 *
	 * @return Strong_Testimonials_Pro_Addons
	 */
	public static function get_instance() {
		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof Strong_Testimonials_Pro_Addons ) ) {
			self::$instance = new Strong_Testimonials_Pro_Addons();
		}

		return self::$instance;
	}

	/**
	 * Get the addons
	 *
	 * @return array
	 */
	private function check_for_addons() {
		$data = get_transient( 'strong_testimonials_licensed_extensions' );

		if ( false !== $data ) {
			$this->addons = is_array( $data ) ? $data : array();
			return;
		}

		// // Make sure this matches the exact URL from your site.
		$url = apply_filters( 'strong_testimonials_addon_server_url', WPMTST_STORE_URL . '/wp-json/mt/v1/get-addons' );
		$url = add_query_arg(
			array(
				'license' => $this->license_key,
				'url'     => $this->site_host,
			),
			$url
		);

		// Get data from the remote URL.
		$response = wp_remote_get( $url );

		if ( ! is_wp_error( $response ) ) {

			// Decode the data that we got.
			$data = json_decode( wp_remote_retrieve_body( $response ), true );

			if ( empty( $data ) ) {
				$this->addons = array( 'not-found' => array() );
			}

			if ( ! empty( $data ) && is_array( $data ) ) {
				$this->addons = $data;
			}

			// Store the data for a week.
			set_transient( 'strong_testimonials_licensed_extensions', $data, 7 * DAY_IN_SECONDS );
		}
	}

	/**
	 * Add scripts for addons page
	 *
	 * @param string $hook
	 */
	public function addons_scripts( $hook ) {

		if ( 'wpm-testimonial_page_strong-testimonials-addons' === $hook ) {
			wp_enqueue_script( 'wpmtst-addon', WPMTST_PRO_URL . 'assets/js/addons.js', array( 'jquery' ), '2.0.0', true );

			$args = array(
				'install_nonce' => wp_create_nonce( 'wpmtst-addon-install' ),
				'connect_error' => esc_html__( 'ERROR: There was an error connecting to the server, Please try again.', 'strong-testimonials-pro' ),
			);
			wp_localize_script( 'wpmtst-addon', 'wpmtstPRO', $args );
		}
	}

	/**
	 * Output the download link for the addons
	 *
	 * @param string $link
	 * @param array  $addon
	 *
	 * @return string
	 */
	public function output_download_link( $link, $addon ) {

		if ( empty( $this->addons ) && '' !== $this->license_key ) {
			$this->check_for_addons();
		}

		$labels = array(
			'install'   => esc_html__( 'Install & Activate', 'strong-testimonials-pro' ),
			'activate'  => esc_html__( 'Activate', 'strong-testimonials-pro' ),
			'installed' => esc_html__( 'Installed', 'strong-testimonials-pro' ),
		);

		$classes = array(
			'install'   => 'button button-primary wpmtst-addon-action',
			'activate'  => 'button button-primary wpmtst-addon-action',
			'installed' => 'button',
		);

		$action = 'install';

		if ( ! isset( $addon['slug'] ) ) {
			$url  = admin_url( 'edit.php?post_type=wpm-testimonial&page=testimonial-settings&tab=licenses' );
			$link = '<a href="' . $url . '" class="button button-primary">' . esc_html__( 'Add license', 'strong-testimonials-pro' ) . '</a>';
			return $link;
		}

		if ( array_key_exists( $addon['slug'], $this->addons ) ) {
			$slug        = $addon['slug'];
			$plugin_path = $slug . '/' . $slug . '.php';

			if ( 'strong-testimonials-imports' === $slug ) {
				$plugin_path = 'strong-testimonials-imports/strong-testimonials-importer.php';
			}

			if ( $this->check_plugin_is_installed( $addon['slug'] ) && ! $this->check_plugin_is_active( $plugin_path ) ) {
				$action = 'activate';
			} elseif ( $this->check_plugin_is_active( $plugin_path ) ) {
				$action = 'installed';
			}

			if ( 'install' !== $action ) {
				$url = $this->create_plugin_link( $action, $plugin_path );
			} else {
				$url = $this->addons[ $addon['slug'] ]['download_link'];
			}

			$attr = '';

			if ( 'installed' !== $action ) {
				$attr = 'data-action="' . $action . '"';
			} else {
				$attr = 'disabled="disabled"';
			}

			$link = '<a href="' . $url . '" ' . $attr . ' class="' . $classes[ $action ] . '">' . $labels[ $action ] . '</a>';
		} elseif ( '' !== $this->license_key ) { //the user has entered a license key, but this extension requires an upgrade
			$url  = WPMTST_STORE_UPGRADE_URL . '?utm_source=st-lite&utm_campaign=upsell&utm_medium=' . $addon['slug'];
			$link = '<a target="_blank" href="' . $url . '" class="button button-primary">' . esc_html__( 'Upgrade', 'strong-testimonials-pro' ) . '</a>';
		} else {
			$url  = admin_url( 'edit.php?post_type=wpm-testimonial&page=testimonial-settings&tab=licenses' );
			$link = '<a href="' . $url . '" class="button button-primary">' . esc_html__( 'Add license', 'strong-testimonials-pro' ) . '</a>';
		}

		return $link;
	}

	/**
	 * Create the plugin link
	 *
	 * @param string $state
	 * @param string $slug
	 *
	 * @return string
	 */
	private function create_plugin_link( $state, $slug ) {
		$string = '';
		// A fix for the importer plugin as it has a different slug. Should respect plugin-name/plugin-name.php but for some reason it doesn't.
		if ( false !== strpos( $slug, 'strong-testimonials-imports.php' ) ) {
			$slug = 'strong-testimonials-imports/strong-testimonials-importer.php';
		}
		switch ( $state ) {
			case 'deactivate':
				$string = add_query_arg(
					array(
						'action'        => 'deactivate',
						'plugin'        => rawurlencode( $slug ),
						'plugin_status' => 'all',
						'paged'         => '1',
						'_wpnonce'      => wp_create_nonce( 'deactivate-plugin_' . $slug ),
					),
					admin_url( 'plugins.php' )
				);
				break;
			case 'activate':
				$string = add_query_arg(
					array(
						'action'        => 'activate',
						'plugin'        => rawurlencode( $slug ),
						'plugin_status' => 'all',
						'paged'         => '1',
						'_wpnonce'      => wp_create_nonce( 'activate-plugin_' . $slug ),
					),
					admin_url( 'plugins.php' )
				);
				break;
			default:
				$string = '';
				break;
		}// End switch().
		return $string;
	}

	private function _get_plugins( $plugin_folder = '' ) {

		if ( ! empty( $this->plugins ) ) {
			return $this->plugins;
		}

		if ( ! function_exists( 'get_plugins' ) ) {
			require_once ABSPATH . 'wp-admin/includes/plugin.php';
		}

		$this->plugins = get_plugins( $plugin_folder );
		return $this->plugins;
	}

	/**
	 * Check if plugin is installed
	 *
	 * @param string $slug
	 *
	 * @return bool
	 */
	private function check_plugin_is_installed( $slug ) {
		if ( file_exists( WP_PLUGIN_DIR . '/' . $slug ) ) {
			return true;
		}
		return false;
	}

	/**
	 * Check if plugin is active
	 *
	 * @return bool
	 */
	private function check_plugin_is_active( $plugin_path ) {
		if ( file_exists( WP_PLUGIN_DIR . '/' . $plugin_path ) ) {
			include_once ABSPATH . 'wp-admin/includes/plugin.php';
			return is_plugin_active( $plugin_path );
		}
	}

	/**
	 * Install addons
	 */
	public function install_addons() {

		// Run a security check first.
		check_admin_referer( 'wpmtst-addon-install', 'nonce' );

		if ( ! current_user_can( 'install_plugins' ) ) {
			echo json_encode( array( 'error' => __( 'There was an error installing the addon. Please try again.', 'strong-testimonials-pro' ) ) );
			die;
		}

		if ( ! isset( $_POST['plugin'] ) ) {
			echo json_encode( array( 'error' => __( 'There was an error installing the addon. Please try again.', 'strong-testimonials-pro' ) ) );
			die;
		}

		$download_url = esc_url( $_POST['plugin'] );
		if ( false === strpos( $download_url, WPMTST_STORE_URL ) ) {
			echo json_encode( array( 'error' => __( 'There was an error installing the addon. Please try again.', 'strong-testimonials-pro' ) ) );
			die;
		}

		global $hook_suffix;

		// Set the current screen to avoid undefined notices.
		set_current_screen();

		// Prepare variables.
		$method = '';
		$url    = add_query_arg(
			array(
				'page' => 'testimonial-settings',
			),
			admin_url( 'admin.php' )
		);
		$url    = esc_url( $url );

		// Start output bufferring to catch the filesystem form if credentials are needed.
		ob_start();
		$creds = request_filesystem_credentials( $url, $method, false, false, null );
		if ( false === $creds ) {
			$form = ob_get_clean();
			echo json_encode( array( 'form' => $form ) );
			die;
		}

		// If we are not authenticated, make it happen now.
		if ( ! WP_Filesystem( $creds ) ) {
			ob_start();
			request_filesystem_credentials( $url, $method, true, false, null );
			$form = ob_get_clean();
			echo json_encode( array( 'form' => $form ) );
			die;
		}

		// We do not need any extra credentials if we have gotten this far, so let's install the plugin.
		require_once ABSPATH . 'wp-admin/includes/class-wp-upgrader.php';
		require_once WPMTST_PRO_PATH . '/includes/class-strong-testimonials-pro-upgrader-skin.php';

		// Create the plugin upgrader with our custom skin.
		$skin      = new Strong_Testimonials_Pro_Upgrader_Skin();
		$installer = new Plugin_Upgrader( $skin );
		$installer->install( $download_url );

		// Flush the cache and return the newly installed plugin basename.
		wp_cache_flush();
		if ( $installer->plugin_info() ) {
			$plugin_basename = $installer->plugin_info();
			$activate_url    = $this->create_plugin_link( 'activate', $plugin_basename );
			echo json_encode(
				array(
					'plugin'       => $plugin_basename,
					'activate_url' => $activate_url,
				)
			);
			die;
		}
	}

	/**
	 * Remove Upgrade to PRO link from plugins if all extensions available
	 */
	public function remove_plugins_upgrade_to_pro( $return_data ) {

		$st_addons  = new Strong_Testimonials_Addons();
		$all_addons = count( $st_addons->get_addons() );
		if ( empty( $this->addons ) && '' !== $this->license_key ) {
			$this->check_for_addons();
		}

		if ( count( $this->addons ) === $all_addons ) {
			return false;
		}

		return $return_data;
	}

	/**
	 * Reload extensions and delete transient
	 */
	public function reload_extensions(){
		delete_transient( 'strong_testimonials_licensed_extensions' );
	}
}

Strong_Testimonials_Pro_Addons::get_instance();
